// filename.h

#ifndef LQ_filename_h
#define LQ_filename_h

#include <sys/types.h> // size_t, bool
#include <dirent.h>    // dirent

#include "cdecls.h"
//#include "types.h" // bool for C

/** \file filename.h
 * file stuff using name as input parameter.  One header for
 * several independent object files.
 */

LQ_BEGIN_CDECLS;

/* 
  filename functions for matching filenames with wildcards.
  
  Gleaned from several GPL'd sourced including xcopy300 and 
  fltk v. 2.0
  License GPL 2+, -rs
*/

/// names may include paths, the 'official' length limit it 
/// two K but this should handle anything realistic.

#define FILENAME_MAXSTR 1024

/// returns non-zero if name matches wildcard pattern.
bool filename_matchExisting(const char *name, const char *pattern);

/** Returns true if any possibly quoted patterns in a comma separated list 
 match.  Each pattern is parsed according to the rules for filename_matchExisting().
 Spaces before or after commas are ignored. -rs
*/

bool filename_matchAnyExisting(const char* name, const char* patterns);


/** Presumes filename_matchExisting() has returned successfuly.
 Sets new name based on wildcard pattern at user 
 supplied buffer and returns non-zero if successful.
 Note: If used to copy or rename files, the user is 
 responsible for checking if anything could be 
 overwritten that shouldn't be.
 */

int filename_matchNew(char *name_out, size_t outlen,
          const char *name_in, const char *pattern);

/// Returns true if the named file exists
bool filename_exists(const char* filename);

/// returns size of file, 0 if it doesn't exist
size_t filename_size(const char* filename);

/// sets buf to absolute path for filename, returns true if successful
bool filename_absolute(char* buf, size_t buflen, const char* filename);

/** Returns true if the file exists and is a directory. */
bool filename_isdir(const char* name);

/**
 *  Returns the modification time of the file as a Unix timestamp
 *  (number of seconds since the start of 1970 in GMT). Returns 0
 *  if the file does not exist.
 */
long filename_mtime(const char *name);

/// loads malloc-ed buffer and set's buffer and size, returns true on 
/// success.  User must free the buffer when done.
bool filename_load(const char* filename, char** pbuf, size_t* pbufsize);

/// saves buffer to filename, returns true if successful.
bool filename_save(const char* filename, char* buf, size_t bufsize);

typedef int dirent_sort_t(const struct dirent **, const struct dirent **);

/// reads directory and returns number of files in plist.
int filename_dirList(const char* dirname, dirent*** plist, dirent_sort_t func);

LQ_END_CDECLS;

#endif // LQ_filename_h

