// filename_stat.cpp - functions using stat

/* 
 * code here is based on code from the long defunct fltk version 2.0
 */

#include <sys/stat.h> // struct stat
#include <string.h>   // strcmp()

// v-1.1.1 -rs
#include <malloc.h>   // free()

#include "filename.h"
#include "cdecls.h"
LQ_BEGIN_CDECLS;

static struct stat last_stat;
static const char *last_statname = 0;
static bool last_result = false;

static bool fill_stat(const char *name) {
  if (last_statname && strcmp(last_statname, name)==0) return last_result;
  
  // // v-1.1.1 -rs
  //delete[] const_cast<char *>( last_statname );
  if(last_statname)
    free((void*)last_statname); 
  last_statname = strdup(name);
  last_result = (stat(name, &last_stat)==0);
  return last_result;
}

/** Returns true if the file exists . */
bool filename_exists(const char* name) {
  return name && *name && fill_stat(name);
}

/** Returns true if the file exists and is a directory. */
bool filename_isdir(const char* name) {
  if (!fill_stat(name)) return false;
  return (last_stat.st_mode & 0170000) == 0040000;
}

/** Returns the size of the file in bytes. Returns zero if it does not exist.*/
size_t filename_size(const char* name) {
  if (!fill_stat(name)) return 0;
  return (size_t)last_stat.st_size;
}

/**
 *  Returns the modification time of the file as a Unix timestamp
 *  (number of seconds since the start of 1970 in GMT). Returns 0
 *  if the file does not exist.
 */
long int filename_mtime(const char *name) {
  if (!fill_stat(name)) return 0L;
  if (last_stat.st_mtime) return (long) last_stat.st_mtime;
  if (last_stat.st_atime) return (long) last_stat.st_atime;
  return (long) last_stat.st_ctime;
}

LQ_END_CDECLS;

